'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = flowReactPropTypes;

var _util = require('./util');

var _convertToPropTypes = require('./convertToPropTypes');

var _convertToPropTypes2 = _interopRequireDefault(_convertToPropTypes);

var _makePropTypesAst = require('./makePropTypesAst');

var _makePropTypesAst2 = _interopRequireDefault(_makePropTypesAst);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var matchedPropTypes = void 0;

// See ExportNamedDeclaration and ImportDeclaration
var importedTypes = {};

function flowReactPropTypes(babel) {
  var t = babel.types;

  var annotate = function annotate(path, matchedPropTypes) {
    var name = path.node.id.name;

    var propTypesAST = (0, _makePropTypesAst2.default)(matchedPropTypes);
    var attachPropTypesAST = t.expressionStatement(t.assignmentExpression('=', t.memberExpression(t.identifier(name), t.identifier('propTypes')), propTypesAST));
    var targetPath = path.parent.type === 'Program' ? path : path.parentPath;
    targetPath.insertAfter(attachPropTypesAST);
  };

  return {
    visitor: {
      Program: function Program(path) {
        matchedPropTypes = null;
        importedTypes = {};
      },
      TypeAlias: function TypeAlias(path) {
        (0, _util.$debug)('TypeAlias found');
        if (!/Props$/.test(path.node.id.name)) {
          (0, _util.$debug)('TypeAlias ' + path.node.id.name + ' is not a Props type');
          return;
        }
        var right = path.node.right;

        if (right.type !== 'ObjectTypeAnnotation') {
          (0, _util.$debug)('Expected ObjectTypeAnnotation but got ' + right.type);
          return;
        }

        var propTypes = (0, _convertToPropTypes2.default)(right, importedTypes);
        matchedPropTypes = propTypes;
      },
      ClassDeclaration: function ClassDeclaration(path) {
        if (!matchedPropTypes) {
          (0, _util.$debug)('at ClassDeclaration no prop TypeAlias was found');
          return;
        } else {
          (0, _util.$debug)('Found ClassDeclaration for the TypeAlias');
        }

        var superClass = path.node.superClass;

        // check if we're extending React.Compoennt

        var extendsReactComponent = superClass && superClass.type === 'MemberExpression' && superClass.object.name === 'React' && superClass.property.name === 'Component';
        var extendsComponent = superClass && superClass.type === 'Identifier' && superClass.name === 'Component';
        if (!extendsReactComponent && !extendsComponent) {
          (0, _util.$debug)('Found a class that isn\'t a react component', superClass);
          return;
        }

        annotate(path, matchedPropTypes);
      },
      FunctionDeclaration: function FunctionDeclaration(path) {
        if (!matchedPropTypes) {
          (0, _util.$debug)('at FunctionDeclaration no prop TypeAlias was found');
          return;
        } else {
          (0, _util.$debug)('Found FunctionDeclaration for the TypeAlias');
        }

        // Check if this looks like a stateless react component:
        var hasPropsParam = path.node.params[0] && path.node.params[0].typeAnnotation && path.node.params[0].typeAnnotation.typeAnnotation && path.node.params[0].typeAnnotation.typeAnnotation.id.name && path.node.params[0].typeAnnotation.typeAnnotation.id.name.endsWith('Props');

        if (!hasPropsParam) return;

        annotate(path, matchedPropTypes);
      },


      // See issue:
      ExportNamedDeclaration: function ExportNamedDeclaration(path) {
        var node = path.node;


        if (!node.declaration || node.declaration.type !== 'TypeAlias') {
          return;
        }
        if (!node.declaration.right.properties) {
          return;
        }

        var propTypes = (0, _convertToPropTypes2.default)(node.declaration.right, importedTypes);
        var propTypesAst = (0, _makePropTypesAst2.default)(propTypes);

        if (propTypesAst.type === 'ObjectExpression') {
          propTypesAst = t.callExpression(t.memberExpression(t.memberExpression(t.callExpression(t.identifier('require'), [t.stringLiteral('react')]), t.identifier('PropTypes')), t.identifier('shape')), [propTypesAst]);
        }

        var exportAst = t.expressionStatement(t.callExpression(t.memberExpression(t.identifier('Object'), t.identifier('defineProperty')), [t.memberExpression(t.identifier('module'), t.identifier('exports')), t.stringLiteral((0, _util.getExportNameForType)(node.declaration.id.name)), propTypesAst]));
        path.insertAfter(exportAst);
      },
      ImportDeclaration: function ImportDeclaration(path) {
        var node = path.node;

        if (node.importKind === 'type') {
          node.specifiers.forEach(function (specifier) {
            var typeName = specifier.type === 'ImportDefaultSpecifier' ? specifier.local.name : specifier.imported.name;

            importedTypes[typeName] = (0, _util.getExportNameForType)(typeName);
            var variableDeclarationAst = t.variableDeclaration('var', [t.variableDeclarator(
            // TODO: use local import name?
            t.identifier((0, _util.getExportNameForType)(typeName)), t.logicalExpression('||', t.memberExpression(t.callExpression(t.identifier('require'), [t.stringLiteral(node.source.value)]), t.identifier((0, _util.getExportNameForType)(typeName))), t.memberExpression(t.memberExpression(t.callExpression(t.identifier('require'), [t.stringLiteral('react')]), t.identifier('PropTypes')), t.identifier('any'))))]);
            path.insertAfter(variableDeclarationAst);
          });
        }
      }
    }
  };
};